@@ -341,10 +341,10 @@ gnome_pilot_conduit_sync_abs_new (char *db_name,
 {
 	GnomePilotConduitSyncAbs *retval;
 
-	retval =  gtk_object_new (gnome_pilot_conduit_sync_abs_get_type (),
-				  "GnomePilotConduitStandard::db_name", db_name,
-				  "GnomePilotConduitStandard::creator_id", creator_id,
-				  NULL);
+	retval =  GNOME_PILOT_CONDUIT_SYNC_ABS (gtk_object_new (gnome_pilot_conduit_sync_abs_get_type (),
+								"GnomePilotConduitStandard::db_name", db_name,
+								"GnomePilotConduitStandard::creator_id", creator_id,
+								NULL));
 
 	return GTK_OBJECT (retval);
 }
