@@ -1,3 +1,4 @@
+
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- *//* 
  * Copyright (C) 1998-2000 Free Software Foundation
  *
@@ -42,6 +43,9 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#define LOCK_DIR "/var/lock"
+#define LOCK_BINARY 0
+
 #ifdef HAVE_SA_LEN
 #ifndef max
 #define max(a,b) ((a) > (b) ? (a) : (b))
@@ -99,10 +103,10 @@ GPilotContext *gpilot_context_new (void)
 	}
 
 	/* get progress stepping, default is -1, if default is returned,
-	   default to five and set it */
+	   default to one and set it */
 	retval->progress_stepping = gnome_config_get_int ("progress_stepping=-1");
 	if (retval->progress_stepping ==  -1) {
-		retval->progress_stepping = 5;
+		retval->progress_stepping = 1;
 		gnome_config_set_int ("progress_stepping", retval->progress_stepping);
 	}
 
@@ -153,6 +157,111 @@ gpilot_context_free (GPilotContext *context)
 	context->devices = NULL;
 }
 
+#define LOCK_DIR "/var/lock"
+
+static int
+gpilot_hdb_uucp_lock (GPilotDevice *device) 
+{
+#ifndef LOCK_BINARY
+	char lock_buffer[12];
+#endif
+	int fd, pid, n;
+	char *p = NULL;
+	char *dev = device->port;
+
+	if (geteuid () != 0) {
+		g_message ("Not root, won't do UUCP lock on %s", device->port);
+		return 1;
+	}
+	
+	if ((p = strrchr(dev, '/')) != NULL) {
+		dev = p + 1;
+	}
+	
+	device->lock_file = g_new0 (gchar, 128);
+
+	snprintf (device->lock_file, 127, "%s/LCK..%s", LOCK_DIR, dev);
+
+	while ((fd = open (device->lock_file, O_EXCL | O_CREAT | O_RDWR, 0644)) < 0) {
+		if (errno != EEXIST) {
+			g_warning ("Can't create lock file %s: %m", device->lock_file);
+			break;
+		}
+		
+		/* Read the lock file to find out who has the device locked. */
+		fd = open (device->lock_file, O_RDONLY, 0);
+		if (fd < 0) {
+			if (errno == ENOENT) /* This is just a timing problem. */
+				continue;
+			g_warning ("Can't open existing lock file %s: %m", device->lock_file);
+			break;
+		}
+
+#ifndef LOCK_BINARY
+		n = read (fd, lock_buffer, 11);
+#else
+		n = read (fd, &pid, sizeof(pid));
+#endif /* LOCK_BINARY */
+		close (fd);
+		fd = -1;
+		if (n <= 0) {
+			g_warning ("Can't read pid from lock file %s", device->lock_file);
+			break;
+		}
+		
+		/* See if the process still exists. */
+#ifndef LOCK_BINARY
+		lock_buffer[n] = 0;
+		pid = atoi (lock_buffer);
+#endif /* LOCK_BINARY */
+		if (pid == getpid ()) {
+			g_warning ("Port %s is already locked", device->port);
+			return 0;           /* somebody else locked it for us */
+		}
+		if (pid == 0
+		    || (kill (pid, 0) == -1 && errno == ESRCH)) {
+			if (unlink (device->lock_file) == 0) {
+				g_message ("Removed stale lock on %s (pid %d)", device->port, pid);
+				continue;
+			}
+			g_warning ("Couldn't remove stale lock on %s", device->port);
+		} else {
+			g_message ("Device %s is locked by pid %d", device->port, pid);
+		}
+		break;
+	}
+
+	if (fd < 0) {
+		g_free (device->lock_file);
+		device->lock_file = NULL;
+		return 0;
+	}
+	
+	pid = getpid ();
+#ifndef LOCK_BINARY
+	snprintf(lock_buffer, 11, "%10d\n", pid);
+	write (fd, lock_buffer, 11);
+#else
+	write (fd, &pid, sizeof (pid));
+#endif
+	close (fd);
+	return 1;
+}
+
+
+static void
+gpilot_hdb_uucp_unlock (GPilotDevice *device) 
+{
+	if (geteuid () != 0) {
+		g_message ("Not root, won't do UUCP unlock on %s", device->port);
+	}
+
+	if (device->lock_file) {
+		unlink(device->lock_file);	
+		g_free (device->lock_file);
+		device->lock_file = NULL;
+	}
+}
 
 /* device stuff next */
 GPilotDevice *
@@ -166,6 +275,10 @@ gpilot_device_new (void)
 static gint 
 gpilot_serial_device_init (GPilotDevice *device)
 {
+	if (!gpilot_hdb_uucp_lock (device)) {
+		return -1;
+	}
+
 	device->fd=open (device->port, O_RDWR|O_NOCTTY|O_NONBLOCK);
 	if (device->fd < 0) {
 		g_warning (_("Could not open device %s (%s): reason: \"%s\"."),
@@ -188,6 +301,8 @@ gpilot_serial_device_init (GPilotDevice *device)
 
 		device->io = NULL;
 
+		gpilot_hdb_uucp_unlock (device);
+	
 		return -1;
 	}
 	device->io = g_io_channel_unix_new (device->fd);
@@ -205,12 +320,25 @@ gpilot_serial_device_free (GPilotDevice *device)
 	}
 	g_free (device->name);
 	g_free (device->port);
+	gpilot_hdb_uucp_unlock (device);
+}
+
+static void
+gpilot_serial_device_deinit (GPilotDevice *device)
+{
+	if (device->io) {
+		g_source_remove (device->in_handle);
+		g_source_remove (device->err_handle);
+		g_io_channel_close (device->io);
+		g_io_channel_unref (device->io);
+	}
 }
 
 #ifdef WITH_USB_VISOR
 static gint 
 gpilot_usb_device_init (GPilotDevice *device)
 {
+	gpilot_hdb_uucp_lock (device);
 	device->fd = -1;
 	device->io = NULL;
 	device->device_exists = FALSE;
@@ -219,6 +347,11 @@ gpilot_usb_device_init (GPilotDevice *device)
 
 static void
 gpilot_usb_device_free (GPilotDevice *device)
+{
+	gpilot_hdb_uucp_unlock (device);
+}
+static void
+gpilot_usb_device_deinit (GPilotDevice *device)
 {
 }
 #endif /* WITH_USB_VISOR */
@@ -235,6 +368,12 @@ gpilot_irda_device_free (GPilotDevice *device)
 {
 	gpilot_serial_device_free (device);
 }
+
+static void
+gpilot_irda_device_deinit (GPilotDevice *device)
+{
+	gpilot_serial_device_deinit (device);
+}
 #endif /* WITH_IRDA */
 
 #ifdef WITH_NETWORK
@@ -248,7 +387,7 @@ gpilot_irda_device_free (GPilotDevice *device)
  * be a demonstration of the proper (or improper, if I'm unlucky) techniques
  * to retrieve networking information.
  */
-void
+static void
 fetch_host (char *hostname, int hostlen, struct in_addr *address,
 	   struct in_addr *mask)
 {
@@ -431,7 +570,7 @@ gpilot_network_device_init (GPilotDevice *device)
 
 /* This is used if the hostname is not correct in the conf file
    I'll move it to the conf file saving later */
-	/* fetch_host (device->host, 128, &address, &netmask); */
+	fetch_host (device->host, 128, &address, &netmask); 
 	
 /* device->host I already have from the config file */
 
@@ -498,6 +637,11 @@ gpilot_network_device_free (GPilotDevice *device)
 	g_free(device->host);
 	g_free(device->netmask);
 }
+
+static void
+gpilot_network_device_deinit (GPilotDevice *device)
+{
+}
 #endif /* WITH_NETWORK */
 
 gint
@@ -580,9 +724,37 @@ gpilot_device_load (GPilotDevice *device, gint i)
 	return result;
 }
 
+void
+gpilot_device_deinit (GPilotDevice *device)
+{
+	g_assert (device != NULL);
+
+	if (device->type == PILOT_DEVICE_SERIAL) {
+		gpilot_serial_device_deinit (device);
+#ifdef WITH_IRDA
+	} else if (device->type == PILOT_DEVICE_IRDA) {
+		gpilot_irda_device_deinit (device);
+#endif /* WITH_IRDA */
+#ifdef WITH_USB_VISOR
+	} else if (device->type == PILOT_DEVICE_USB_VISOR) {
+		gpilot_usb_device_deinit (device);
+#endif /* WITH_USB_VISOR */
+#ifdef WITH_NETWORK
+	} else if (device->type == PILOT_DEVICE_NETWORK) {
+		gpilot_network_device_deinit (device);
+#endif /* WITH_NETWORK */
+	} else {
+		g_warning (_("Unknown device type"));
+	}
+}
+
 void
 gpilot_device_free (GPilotDevice *device)
 {
+	g_assert (device != NULL);
+
+	gpilot_device_deinit (device);
+
 	if (device->type == PILOT_DEVICE_SERIAL) {
 		gpilot_serial_device_free (device);
 #ifdef WITH_IRDA
@@ -600,13 +772,6 @@ gpilot_device_free (GPilotDevice *device)
 	} else {
 		g_warning (_("Unknown device type"));
 	}
-	
-	if (device->io) {
-		g_source_remove (device->in_handle);
-		g_source_remove (device->err_handle);
-		g_io_channel_close (device->io);
-		g_io_channel_unref (device->io);
-	}
 
 	g_free (device);
 }
