@@ -46,6 +46,7 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <pwd.h>
+#include <unistd.h>
 
 #include "orbit_daemon_glue.h"
 #include "gpilot-gui.h"
@@ -114,8 +115,9 @@ pilot_connect (GPilotDevice *device,int *error)
 	int sd;
 	int ret;
     
-	if (device->type != PILOT_DEVICE_NETWORK)
+	if (device->type != PILOT_DEVICE_NETWORK) {
 		pilot_set_baud_rate (device);
+	}
 
 	if (!(sd = pi_socket (PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
 		g_warning ("pi_socket: %s",strerror (errno));
@@ -124,23 +126,19 @@ pilot_connect (GPilotDevice *device,int *error)
 	}
 	
 	addr.pi_family = PI_AF_SLP;
-	if (device->port && strlen (device->port)>13) {
-		g_warning ("Ooops, pilot-link cannot handle ports with names longer then 13 chars.");
-		g_warning ("Sugggested fix is to make a symbolic link with a shorten name and");
-		g_warning ("Use that instead");
-		g_error ("exiting, pending fix...");
-	}
-	strcpy (addr.pi_device,device->port);
 
 /*
 	Most important for networking
 	. resolves inside libpisock as network
 	It is done earlier in gpilotd_device_init
 	so don't really need to do it again here.
-
-	if (device->type == PILOT_DEVICE_NETWORK)
-		device->port = ".";
 */
+	if (device->type == PILOT_DEVICE_NETWORK) {
+		device->port = ".";
+	}
+
+	strcpy (addr.pi_device,device->port);
+
 	ret = pi_bind (sd, (struct sockaddr*)&addr, sizeof (addr));
 	if (ret == -1) {
 		g_warning (_("Unable to bind to pilot"));
@@ -162,6 +160,7 @@ pilot_connect (GPilotDevice *device,int *error)
 		if (error) *error = 2;
 		return 0;
 	}
+
 	if (error) *error = 0;
 	return sd;
 }
@@ -220,38 +219,6 @@ match_pilot_and_majick (const GPilotPilot *pilot,
 
 /**************************/
 
-
-/* 
- * Start restoring a pilot profile, using the given (open hopefully)
- * filedescriptor, given a device (in case we need it again) and the
- * pilot info structure
- */
-
-static gboolean
-gpilot_start_restore (int pfd, 
-		      GPilotDevice *device,
-		      GPilotPilot *pilot)
-{
-	GnomePilotConduitManagement *gpcm;
-	GnomePilotConduit *conduit;
-	int err;
-	gboolean result = TRUE;
-
-	gpcm = gnome_pilot_conduit_management_new ("gpbackup1", GNOME_PILOT_CONDUIT_MGMT_ID);
-	err = gnome_pilot_conduit_management_instantiate_conduit (gpcm,
-								  pilot,
-								  &conduit);
-	if (err == GNOME_PILOT_CONDUIT_MGMT_OK && GNOME_IS_PILOT_CONDUIT_BACKUP (conduit)) {
-		gnome_pilot_conduit_backup_restore (GNOME_PILOT_CONDUIT_BACKUP (conduit),
-						    pfd, 
-						    NULL);
-	} else {
-		result = FALSE;
-	}
-							    
-	return result;
-}
-
 /*
  * We get a pfd, a device and the context (called if a pilot
  * with id == 0 synchronizes)
@@ -261,7 +228,8 @@ gpilot_start_restore (int pfd,
  */
 
 static gboolean
-gpilot_attempt_restore (int pfd, 
+gpilot_attempt_restore (struct PilotUser pu,
+			int pfd, 
 			GPilotDevice *device, 
 			GPilotContext *context)
 {
@@ -292,7 +260,9 @@ gpilot_attempt_restore (int pfd,
 		GPilotPilot *a_pilot;
 		a_pilot = gpilot_gui_restore (context, pilot);
 		if (a_pilot) {
-			result = gpilot_start_restore (pfd, device, a_pilot);
+			orbed_notify_connect (pilot->name,pu);				
+			result = gpilot_start_unknown_restore (pfd, device, a_pilot);
+			orbed_notify_disconnect (pilot->name);
 		}
 	} else {
 		/* MUST GO */
@@ -303,7 +273,7 @@ gpilot_attempt_restore (int pfd,
 					   cardinfo.creation,
 					   sysinfo.romVersion);
 	}
-	pilot_disconnect (pfd);		
+
 	return TRUE;
 }
 
@@ -324,13 +294,12 @@ gpilot_syncing_unknown_pilot (struct PilotUser pu,
 	   has password set
 	   bug # 8217 */
 	if (pu.userID == 0) {
-		if (gpilot_attempt_restore (pfd, device, context) == FALSE) {
+		if (gpilot_attempt_restore (pu, pfd, device, context) == FALSE) {
 			gpilot_gui_warning_dialog (_("Use gnomecc to configure pilot"));
 		}
 	} else {
 		/* FIXME: here we should offer to create a profile for the pilot,
 		   bug # 8218 */
-		pilot_disconnect (pfd);
 		gpilot_gui_warning_dialog (_("Unknown pilot - no pilots matches ID %ld\n"
 					     "Use gnomecc to set pilot's ID"),pu.userID);
 	}
@@ -355,7 +324,7 @@ do_cradle_events (int pfd,
 	   in orbed_notify_completion */
 	events = gpc_queue_load_requests_for_cradle (device->name);
 
-	g_message (_("Cradle %s has %d events"),device->name,g_list_length (events));
+	g_message (_("Cradle %s has %d events"), device->name, g_list_length (events));
 
 	/* if no events, return FALSE */
 	if (g_list_length (events)==0) ret = FALSE;
@@ -368,7 +337,7 @@ do_cradle_events (int pfd,
 		switch (req->type) {
 		case GREQ_SET_USERINFO:
 			g_message (_("Setting userinfo..."));
-			g_snprintf (pu->username,127,"%s",req->parameters.set_userinfo.user_id);
+			g_snprintf (pu->username,127,"%s", req->parameters.set_userinfo.user_id);
 			pu->userID = req->parameters.set_userinfo.pilot_id;
 			dlp_WriteUserInfo (pfd,pu);
 			if (req->parameters.set_userinfo.continue_sync) {
@@ -569,8 +538,6 @@ gpilot_syncing_known_pilot (GPilotPilot *pilot,
 	struct stat buf; 
 	int ret;
 	
-	orbed_notify_connect (pilot->name,pu);
-	
 	ret=stat (pilot->sync_options.basedir, &buf); 
 	if (ret < 0 || !( S_ISDIR (buf.st_mode) && (buf.st_mode & (S_IRUSR | S_IWUSR |S_IXUSR))) ) {
 		
@@ -579,7 +546,7 @@ gpilot_syncing_known_pilot (GPilotPilot *pilot,
 					     "Please fix it or use gnomecc to choose another directory."),
 					   pilot->sync_options.basedir);	
 	} else {
-		
+		gboolean pwd_ok = TRUE;
 		/* If pilot has password, check against the encrypted version
 		   on the pilot */
 		if (pilot->passwd) {
@@ -587,18 +554,17 @@ gpilot_syncing_known_pilot (GPilotPilot *pilot,
 			pwd = (char*)g_malloc (pu.passwordLength);
 			strncpy (pwd,pu.password,pu.passwordLength);
 			if (g_strcasecmp (pilot->passwd,(char*)crypt (pwd,pilot->passwd))) {
-				pilot_disconnect (pfd);
-				orbed_notify_disconnect (pilot->name);
+				pwd_ok = FALSE;
 				gpilot_gui_warning_dialog (_("Unknown pilot - no pilots matches ID %ld\n"
 							     "Use gnomecc to set pilot's ID"),pu.userID);
 			}
 			g_free (pwd);
 		} 
 		
-		do_sync (pfd,context,&pu,pilot,device);
+		if (pwd_ok) {
+			do_sync (pfd,context,&pu,pilot,device);
+		}
 	}
-	pilot_disconnect (pfd);
-	orbed_notify_disconnect (pilot->name);
 }
 
 /*
@@ -640,24 +606,27 @@ sync_device (GPilotDevice *device, GPilotContext *context)
 		if (dlp_ReadUserInfo (pfd,&pu) < 0) {
 			/* no ? drop connection then */
 			g_warning (_("An error occured while getting the pilot's user data"));
-			pilot_disconnect (pfd);
 		} else {
+	
 			/* If there are cradle specific events, handle them and stop */
 			if (do_cradle_events (pfd,context,&pu,device)) {
 				g_message (_("Completed events for cradle %s (%s)"),device->name,device->port);
-				pilot_disconnect (pfd);
 			} else {
 				/* No cradle events, validate pilot */
 				pilot = gpilot_find_pilot_by_id (pu.userID,context->pilots);
+
 				if (pilot == NULL) {
 					/* Pilot is not known */
 					gpilot_syncing_unknown_pilot (pu, pfd, device, context);
 				} else {
 					/* Pilot is known, make connect notifications */
+					orbed_notify_connect (pilot->name,pu);				
 					gpilot_syncing_known_pilot (pilot, pu, pfd, device, context);
-				}
+					orbed_notify_disconnect (pilot->name);
+				}				
 			}
 		}
+		pilot_disconnect (pfd);
 		close (pfd);
 	} else {
 		if (connect_error==1) return FALSE; /* remove this device */
@@ -696,7 +665,7 @@ device_in (GIOChannel *io_channel,
 #ifdef WITH_IRDA
 	if (device->type == PILOT_DEVICE_IRDA) {
 		g_message ("Restarting irda funk...");
-		gpilot_device_free (device);
+		gpilot_device_deinit (device);
 		gpilot_device_init (device);
 		monitor_channel (device, context);
 		result = FALSE;
@@ -910,6 +879,7 @@ void monitor_channel (GPilotDevice *dev,GPilotContext *context) {
 						 (GIOFunc)device_err,
 						 (gpointer)context);
 	} else if (dev->type == PILOT_DEVICE_NETWORK) {
+#ifdef WITH_NETWORK
 		dev->in_handle = g_io_add_watch (dev->io,
 						G_IO_IN,
 						(GIOFunc)network_device_in,
@@ -918,6 +888,9 @@ void monitor_channel (GPilotDevice *dev,GPilotContext *context) {
 						 G_IO_ERR|G_IO_PRI|G_IO_HUP|G_IO_NVAL,
 						 (GIOFunc)network_device_err,
 						 (gpointer)context);
+#else /* WITH_NETWORK */
+		g_assert_not_reached ();
+#endif /* WITH_NETWORK */
 	} if (dev->type == PILOT_DEVICE_USB_VISOR) {
 #ifdef WITH_USB_VISOR
 #ifdef linux
@@ -944,13 +917,15 @@ void monitor_channel (GPilotDevice *dev,GPilotContext *context) {
 						(GIOFunc)visor_devices_err,
 						(gpointer)context);
 		}
-#endif		
-#endif
+#else /* linux*/
+		g_assert_not_reached ();
+#endif /* linux */
+#endif /* WITH_USB_VISOR */
 		dev->device_exists = FALSE;
 	}
 
 	if (dev->type == PILOT_DEVICE_NETWORK) {
-		g_message (_("Listening on %s (%s)"),dev->ip, dev->host);
+		g_message (_("Watching %s (%s, %s)"), dev->name, dev->ip, dev->host);
 	} else {
 		g_message (_("Watching %s (%s)"),dev->name,dev->port);
 	}
@@ -1058,7 +1033,7 @@ wait_for_sync_and_sync (GPilotContext *context) {
 	signal (SIGINT,sig_int_handler);
 	signal (SIGHUP,sig_hup_handler);
 
-	g_list_foreach (context->devices,(GFunc)monitor_channel,context);
+	g_list_foreach (context->devices, (GFunc)monitor_channel, context);
 
 	while (1) {
 		if (reread_config) {
